@@ -1,3 +1,4 @@
+#include <gnome.h>
 #include <stdlib.h>
 #include <stdio.h>
 #include <unistd.h>
@@ -22,6 +23,11 @@ GList *newGhelpTable(struct _toc_config *conf)
     GList *list = NULL;
     struct _big_table_entry *entry;
     char *lang;
+    GList *lang_list = NULL;
+    GList *temp = NULL;
+    struct stat buf;
+
+    lang_list = gnome_i18n_get_language_list ("LC_MESSAGE");
 
     while (conf->path) {
 	if (conf->type != TOC_GHELP_TYPE) {
@@ -37,28 +43,32 @@ GList *newGhelpTable(struct _toc_config *conf)
 		    continue;
 		}
 			
-		lang = getenv("LANGUAGE");
-		if (!lang)
-		    lang = getenv("LANG");
-		if (!lang)
-		    lang = "C";
-		sprintf(filename, "%s/%s/%s/index.html",
+		temp= lang_list;
+		while (temp)
+		  {
+		    lang= (gchar*) temp->data;
+		    sprintf (filename, "%s/%s/%s/index.html",
 			conf->path, dirp->d_name, lang);
+		    
+		    if (stat (filename, &buf) == 0)
+		      break;
+
+		    temp= temp->next;
+		  }
 		
-		/* XXX need to traverse LANGUAGE to find right
-		   topic.dat. In fact, this LANG stuff doesn't really
-		   belong here at all */
-
-		entry = g_malloc(sizeof(*entry));
-		entry->type = TOC_GHELP_TYPE;
-		entry->name = g_strdup(dirp->d_name);
-		entry->section = NULL;
-		entry->expanded = 1;
-		entry->ext = 0;
-		entry->filename = g_strdup(filename);
-
-		list = g_list_insert_sorted(list, entry,
-					    (GCompareFunc)compareItems);
+		if (temp)
+		  {
+		    entry = g_malloc(sizeof(*entry));
+		    entry->type = TOC_GHELP_TYPE;
+		    entry->name = g_strdup(dirp->d_name);
+		    entry->section = NULL;
+		    entry->expanded = 1;
+		    entry->ext = 0;
+		    entry->filename = g_strdup(filename);
+		    
+		    list = g_list_insert_sorted(list, entry,
+						(GCompareFunc)compareItems);
+		  }
 	    }
 	    closedir(d);
 	}
