@@ -29,9 +29,10 @@ transport( docObj obj, DataCache cache )
     gint  rc;
     gint len;
 
-    if (cache) {
-	url = docObjGetDecomposedUrl(obj);
-	sprintf(key, "%s://%s%s", url->access, url->host, url->path);
+    url = docObjGetDecomposedUrl(obj);
+    sprintf(key, "%s://%s%s", url->access, url->host, url->path);
+
+    if (docObjUseCache(obj) && cache) {
 	p = lookupInDataCacheWithLen(cache, key, &len);
 	if (p) {
 	    g_message("cache hit: %s", key);
@@ -53,7 +54,7 @@ transport( docObj obj, DataCache cache )
 	docObjGetRawData(obj, &p, &len);
 	copy = g_malloc(len);
 	memcpy(copy, p, len);
-	addToDataCache(cache, key, copy, len);
+	addToDataCache(cache, key, copy, len, ! docObjUseCache(obj));
     }
     
     return 0;
@@ -144,12 +145,12 @@ transportHTTP( docObj obj )
     
     if (isdigit(hostname[0])) {
 	if (!inet_aton(hostname, &serverAddress)) {
-	    g_error("unable to resolve host: %s", hostname);
+	    g_warning("unable to resolve host: %s", hostname);
 	    return -1;
 	}
     } else {
 	if (getHostByName(hostname, &serverAddress)) {
-	    g_error("unable to resolve host: %s", hostname);
+	    g_warning("unable to resolve host: %s", hostname);
 	    return -1;
 	}
     }
@@ -161,7 +162,7 @@ transportHTTP( docObj obj )
     sock = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);
 
     if (connect(sock, (struct sockaddr *) &destPort, sizeof(destPort))) {
-	g_error("unable to connect to host: %s", hostname);
+	g_warning("unable to connect to host: %s", hostname);
 	return -1;
     }
 
