@@ -9,16 +9,15 @@
 #include <config.h>
 #include <stdio.h>
 #include <sys/stat.h>
-#include <dlfcn.h>
-#include <unistd.h>
 #include <dirent.h>
+#include <unistd.h>
 #include <string.h>
 #include <gnome.h>
 #include "panel-widget.h"
 #include "panel.h"
 #include "panel_config_global.h"
 #include "menu.h"
-
+#include "mico-glue.h"
 
 #define SMALL_ICON_SIZE 20
 #define BIG_ICON_SIZE   48
@@ -35,10 +34,35 @@ int show_small_icons = TRUE;
 
 extern GlobalConfig global_config;
 
-typedef struct {
-	char *translated;
-	char *original_id;
-} AppletItem;
+
+/*the most important dialog in the whole application*/
+void
+about_cb (GtkWidget *widget, gpointer data)
+{
+	GtkWidget *about;
+	gchar *authors[] = {
+	  "George Lebl (jirka@5z.com)",
+	  "Miguel de Icaza (miguel@kernel.org)",
+	  "Federico Mena (quartic@gimp.org)",
+	  "Tom Tromey (tromey@cygnus.com)",
+	  "Ian Main (slow@intergate.bc.ca)",
+	  "and finally, The Knights Who Say ... NI!",
+	  NULL
+	  };
+
+	about = gnome_about_new ( "The GNOME Panel", VERSION,
+			"(C) 1998 the Free Software Foundation",
+			authors,
+			"This program is responsible for launching "
+			"other applications, embedding small applets "
+			"within itself, world peace, and random X crashes.",
+			NULL);
+	gtk_widget_show (about);
+
+	return;
+}
+
+
 
 
 void
@@ -52,7 +76,7 @@ activate_app_def (GtkWidget *widget, void *data)
 static void
 kill_small_icon(GtkWidget *widget, gpointer data)
 {
-	if(small_icons) g_list_remove(small_icons,widget);
+	if(small_icons) small_icons = g_list_remove(small_icons,widget);
 }
 
 void
@@ -77,6 +101,7 @@ setup_menuitem (GtkWidget *menuitem, GtkWidget *pixmap, char *title)
 	if (pixmap) {
 		gtk_container_add (GTK_CONTAINER (align), pixmap);
 		gtk_widget_set_usize (align, 22, 16);
+		gtk_widget_show (pixmap);
 	} else
 		gtk_widget_set_usize (align, 22, 16);
 
@@ -94,7 +119,8 @@ free_app_def (GtkWidget *widget, void *data)
 {
 	GnomeDesktopEntry *item = data;
 
-	gnome_desktop_entry_free (item);
+	if(item)
+		gnome_desktop_entry_free (item);
 }
 
 void
@@ -113,28 +139,54 @@ free_string (GtkWidget *widget, void *data)
 	g_free (data);
 }
 
-void
-add_to_panel (char *applet, char *arg)
+static gint
+add_to_panel (char *applet, char *path, char *arg)
 {
-	load_applet(applet,arg,PANEL_UNKNOWN_APPLET_POSITION,0);
+	load_applet(applet,path,arg,
+		    PANEL_UNKNOWN_APPLET_POSITION,0,NULL);
+	return TRUE;
 }
 
-void
+static gint
 add_app_to_panel (GtkWidget *widget, void *data)
 {
 	GnomeDesktopEntry *ii = data;
 
-	add_to_panel ("Launcher", ii->location);
+	return add_to_panel (LAUNCHER_ID, NULL, ii->location);
 }
 
-void
+static gint
 add_dir_to_panel (GtkWidget *widget, void *data)
 {
-	add_to_panel (MENU_ID, data);
+	return add_to_panel (MENU_ID, NULL, data);
+}
+
+
+static gint
+add_applet (GtkWidget *w, gpointer data)
+{
+	GnomeDesktopEntry *ii = data;
+	char *path;
+	char *param;
+	gint r;
+
+	path = ii->exec[0];
+
+	g_return_val_if_fail(path!=NULL,FALSE);
+	
+	if((ii->exec[1])!=NULL)
+		param = gnome_string_joinv (" ", ii->exec + 1);
+	else
+		param = NULL;
+
+	r = add_to_panel(EXTERN_ID,path,param);
+
+	if(param) g_free(param);
+	return r;
 }
 
 static GtkWidget *
-create_menu_at (GtkWidget *window, char *menudir, int create_app_menu)
+create_menu_at (char *menudir, int create_app_menu, int applets)
 {	
 	GnomeDesktopEntry *item_info;
 	GtkWidget *menu;
@@ -173,8 +225,8 @@ create_menu_at (GtkWidget *window, char *menudir, int create_app_menu)
 		if (S_ISDIR (s.st_mode)) {
 			char *dentry_name;
 
-			sub = create_menu_at (window, filename,
-					      create_app_menu);
+			sub = create_menu_at (filename, create_app_menu,
+					      applets);
 			if (!sub) {
 				g_free (filename);
 				continue;
@@ -261,8 +313,15 @@ create_menu_at (GtkWidget *window, char *menudir, int create_app_menu)
 		gtk_signal_connect (GTK_OBJECT (menuitem), "destroy",
 				    (GtkSignalFunc) free_app_def, item_info);
 
-		activate_func = create_app_menu ? (GtkSignalFunc) add_app_to_panel : (GtkSignalFunc) activate_app_def;
-		gtk_signal_connect (GTK_OBJECT (menuitem), "activate", activate_func, item_info);
+		if(item_info && item_info->exec) {
+			activate_func = create_app_menu ?
+				GTK_SIGNAL_FUNC(add_app_to_panel) :
+				GTK_SIGNAL_FUNC(activate_app_def);
+			if(applets && activate_app_def)
+				activate_func = GTK_SIGNAL_FUNC(add_applet);
+			gtk_signal_connect (GTK_OBJECT (menuitem), "activate",
+					    activate_func, item_info);
+		}
 
 		g_free (filename);
 	}
@@ -286,19 +345,19 @@ menu_position (GtkMenu *menu, gint *x, gint *y, gpointer data)
 	gdk_window_get_origin (widget->window, &wx, &wy);
 
 	switch(menup->orient) {
-		case MENU_DOWN:
+		case ORIENT_DOWN:
 			*x = wx;
 			*y = wy + widget->allocation.height;
 			break;
-		case MENU_UP:
+		case ORIENT_UP:
 			*x = wx;
 			*y = wy - GTK_WIDGET (menu)->allocation.height;
 			break;
-		case MENU_RIGHT:
+		case ORIENT_RIGHT:
 			*x = wx + widget->allocation.width;
 			*y = wy;
 			break;
-		case MENU_LEFT:
+		case ORIENT_LEFT:
 			*x = wx - GTK_WIDGET (menu)->allocation.width;
 			*y = wy;
 			break;
@@ -313,21 +372,45 @@ menu_position (GtkMenu *menu, gint *x, gint *y, gpointer data)
 	if(*y < 0) *y =0;
 }
 
-void
+static void
 destroy_menu (GtkWidget *widget, gpointer data)
 {
 	Menu *menu = data;
 
 	if(menu->menu != root_menu)
 		gtk_widget_unref(menu->menu);
+	else 
+		/*this will disconnect the "deactivate" signal*/
+		gtk_signal_disconnect_by_data(GTK_OBJECT(root_menu),menu);
 	g_free(menu);
 }
 
+static void
+menu_deactivate(GtkWidget *w, gpointer data)
+{
+	Menu *menu = data;
+	PanelWidget *panel =
+		gtk_object_get_data(GTK_OBJECT(menu->button->parent),
+				    PANEL_APPLET_PARENT_KEY);
+	/* allow the panel to hide again */
+	panel->autohide_inhibit = FALSE;
+}
+
+
+
 static gint
 menu_button_press(GtkWidget *widget, GdkEventButton *event, gpointer data)
 {
 	Menu *menu = data;
 	if(event->button==1) {
+		PanelWidget *panel =
+			gtk_object_get_data(GTK_OBJECT(menu->button->parent),
+					    PANEL_APPLET_PARENT_KEY);
+		/*so that the panel doesn't pop down until we're done with
+		  the menu */
+		panel->autohide_inhibit = TRUE;
+		panel_widget_queue_pop_down(panel);
+
 		gtk_menu_popup(GTK_MENU(menu->menu), 0,0, menu_position,
 			       data, event->button, event->time);
 		return TRUE;
@@ -341,127 +424,96 @@ panel_configure (GtkWidget *widget, void *data)
 	panel_config_global();
 }
 
-static AppletItem *
-applet_item_new(char *translated, char *original_id)
-{
-	AppletItem *ai;
-
-	ai = g_new(AppletItem, 1);
-	ai->translated = translated;
-	ai->original_id = original_id;
-
-	return ai;
-}
-
-static void
-applet_item_destroy(AppletItem *ai)
-{
-	g_free(ai->translated);
-	g_free(ai->original_id);
-	g_free(ai);
-}
-
 static void
 add_applet_to_panel_data(GtkWidget *widget, gpointer data)
 {
-	add_to_panel((char *)data, NULL);
+	add_to_panel((char *)data, NULL, NULL);
+}
+
+static gint
+act_really_add_swallow(GtkWidget *w, gpointer data)
+{
+	GtkWidget *entry = data;
+	GtkWidget *d = gtk_object_get_user_data(GTK_OBJECT(entry));
+
+	gtk_widget_hide(d);
+	add_to_panel(SWALLOW_ID, NULL, gtk_entry_get_text(GTK_ENTRY(entry)));
+
+	return TRUE;
+}
+
+static gint
+really_add_swallow(GtkWidget *d,gint button, gpointer data)
+{
+	GtkWidget *entry = data;
+
+	gtk_widget_hide(d);
+	if(button == 0)
+		add_to_panel(SWALLOW_ID, NULL,
+			     gtk_entry_get_text(GTK_ENTRY(entry)));
+	return TRUE;
 }
 
 static void
-add_applet_to_panel(GtkWidget *widget, gpointer data)
+add_swallow_to_panel(GtkWidget *widget, gpointer data)
 {
-	add_to_panel(gtk_object_get_user_data(GTK_OBJECT(widget)), NULL);
+	static GtkWidget *d=NULL;
+
+	if(!d) {
+		GtkWidget *w;
+		GtkWidget *box;
+		d = gnome_dialog_new(_("Create swallow applet"),
+				     GNOME_STOCK_BUTTON_OK,
+				     GNOME_STOCK_BUTTON_CANCEL,
+				     NULL);
+		box = gtk_hbox_new(FALSE,5);
+		gtk_widget_show(box);
+		gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(d)->vbox),box,
+				   TRUE,TRUE,5);
+
+		w = gtk_label_new(_("Title of application to swallow"));
+		gtk_widget_show(w);
+		gtk_box_pack_start(GTK_BOX(box),w,TRUE,TRUE,5);
+		w = gtk_entry_new();
+		gtk_widget_show(w);
+		gtk_box_pack_start(GTK_BOX(box),w,TRUE,TRUE,5);
+
+		gtk_object_set_user_data(GTK_OBJECT(w),d);
+
+		gtk_signal_connect(GTK_OBJECT(d),"clicked",
+				   GTK_SIGNAL_FUNC(really_add_swallow),
+				   w);
+		gtk_signal_connect(GTK_OBJECT(w),"activate",
+				   GTK_SIGNAL_FUNC(act_really_add_swallow),
+				   w);
+
+		gnome_dialog_close_hides(GNOME_DIALOG(d),TRUE);
+
+		gnome_dialog_set_default(GNOME_DIALOG(d),0);
+
+		gtk_widget_grab_focus(w);
+	}
+	gtk_widget_show(d);
 }
 
-static void
-munge_applet_item(gpointer untrans, gpointer user_data)
-{
-	GList      **list;
-	GList       *node;
-	int          pos;
-	char        *trans;
-
-	list = user_data;
-
-	/* Insert applet id in alphabetical order */
-	
-	node = *list;
-	pos  = 0;
-
-	trans = _(untrans);
-	
-	for (pos = 0; node; node = node->next, pos++)
-		if (strcmp(trans, _(node->data)) < 0)
-			break;
-
-	*list = g_list_insert(*list,
-			      applet_item_new(g_strdup(trans), untrans),
-			      pos);
-}
-
-static void
-append_applet_item_to_menu(gpointer data, gpointer user_data)
-{
-	GtkMenu    *menu;
-	GtkWidget  *menuitem;
-	AppletItem *ai;
-	char       *oid;
-
-	ai = data;
-	menu = GTK_MENU(user_data);
-
-	oid = g_strdup(ai->original_id);
-
-	menuitem = gtk_menu_item_new();
-	setup_menuitem(menuitem, NULL, ai->translated);
-	gtk_object_set_user_data(GTK_OBJECT(menuitem), oid);
-	gtk_signal_connect(GTK_OBJECT(menuitem), "activate",
-			   (GtkSignalFunc) add_applet_to_panel,
-			   NULL);
-	gtk_signal_connect(GTK_OBJECT(menuitem), "destroy",
-			   (GtkSignalFunc) free_string,
-			   oid);
-	
-	gtk_menu_append(menu, menuitem);
-
-	/* Free applet item */
-
-	applet_item_destroy(ai);
-}
 
 static GtkWidget *
 create_applets_menu(void)
 {
-	GtkWidget    *menu;
-	GList        *list;
-	GList        *applets_list;
+	GtkWidget *applet_menu;
+	char *menu_base = gnome_unconditional_datadir_file ("applets");
+	char *menudir;
 
-	return gtk_menu_new();
+	menudir = g_concat_dir_and_file (menu_base, ".");
+	g_free (menu_base);
+	if (!g_file_exists (menudir)) {
+		g_free (menudir);
+		return NULL;
+	}
 
-	/* Get list of applet types */
-
-	/*FIXME:  */
-	list = NULL;
-
-	/* Now translate and sort them */
-
-	applets_list = NULL;
-	g_list_foreach(list, munge_applet_item, &applets_list);
-
-	/* Create a menu of the translated and sorted ones */
-
-	g_list_free(list);
-
-	menu = gtk_menu_new();
-
-	g_list_foreach(applets_list, append_applet_item_to_menu, menu);
-
-	/* Destroy the list (the list items have already been freed by
-	 * append_applet_item_to_menu()), and return the finished menu.
-	 */
-
-	g_list_free(applets_list);
-	return menu;
+	applet_menu = create_menu_at(menudir,0,TRUE);
+	g_free (menudir);
+	return applet_menu;
 }
 
 static GtkWidget *
@@ -476,12 +528,11 @@ create_panel_submenu (GtkWidget *app_menu)
 	gtk_menu_append (GTK_MENU (menu), menuitem);
 	gtk_menu_item_set_submenu (GTK_MENU_ITEM (menuitem), app_menu);
 
-	/*FIXME: */
-	/*menuitem = gtk_menu_item_new ();
+	menuitem = gtk_menu_item_new ();
 	setup_menuitem (menuitem, 0, _("Add applet"));
 	gtk_menu_append (GTK_MENU (menu), menuitem);
 	gtk_menu_item_set_submenu (GTK_MENU_ITEM (menuitem),
-		create_applets_menu());*/
+		create_applets_menu());
 
 	add_menu_separator(menu);
 
@@ -492,28 +543,57 @@ create_panel_submenu (GtkWidget *app_menu)
 			   (GtkSignalFunc) add_applet_to_panel_data,
 			   MENU_ID);
 
+	menuitem = gtk_menu_item_new ();
+	setup_menuitem (menuitem, 0, _("Add log out button"));
+	gtk_menu_append (GTK_MENU (menu), menuitem);
+	gtk_signal_connect(GTK_OBJECT(menuitem), "activate",
+			   (GtkSignalFunc) add_applet_to_panel_data,
+			   LOGOUT_ID);
+
+#ifdef _DRAWER_
 	menuitem = gtk_menu_item_new ();
 	setup_menuitem (menuitem, 0, _("Add drawer"));
 	gtk_menu_append (GTK_MENU (menu), menuitem);
 	gtk_signal_connect(GTK_OBJECT(menuitem), "activate",
 			   (GtkSignalFunc) add_applet_to_panel_data,
 			   DRAWER_ID);
+#endif
+
+#ifdef _SWALLOW_
+	menuitem = gtk_menu_item_new ();
+	setup_menuitem (menuitem, 0, _("Add swallowed app"));
+	gtk_menu_append (GTK_MENU (menu), menuitem);
+	gtk_signal_connect(GTK_OBJECT(menuitem), "activate",
+			   (GtkSignalFunc) add_swallow_to_panel,NULL);
+#endif
 
 	add_menu_separator(menu);
 	
 	menuitem = gtk_menu_item_new ();
-	setup_menuitem (menuitem, 0, _("Global properties..."));
+	setup_menuitem (menuitem,
+			gnome_stock_pixmap_widget(menu,
+					  GNOME_STOCK_PIXMAP_PREFERENCES),
+			_("Global properties..."));
 	gtk_menu_append (GTK_MENU (menu), menuitem);
         gtk_signal_connect (GTK_OBJECT (menuitem), "activate",
 			    (GtkSignalFunc) panel_configure, 0);
 
+	menuitem = gtk_menu_item_new ();
+	setup_menuitem (menuitem,
+			gnome_stock_pixmap_widget(menu,
+						  GNOME_STOCK_PIXMAP_ABOUT),
+			_("About..."));
+	gtk_menu_append (GTK_MENU (menu), menuitem);
+	gtk_signal_connect (GTK_OBJECT (menuitem), "activate",
+			    (GtkSignalFunc) about_cb, 0);
+
 	return menu;
 }
 
 static void
 panel_lock (GtkWidget *widget, void *data)
 {
-	system ("gnome-lock");
+	system ("(xlock&)");
 }
 
 static void
@@ -544,35 +624,39 @@ add_special_entries (GtkWidget *menu, GtkWidget *app_menu)
 	gtk_signal_connect (GTK_OBJECT (menuitem), "activate", (GtkSignalFunc) panel_lock, 0);
 
 	menuitem = gtk_menu_item_new ();
-	setup_menuitem (menuitem, 0, _("Log out"));
+	setup_menuitem (menuitem,
+			gnome_stock_pixmap_widget(menu,
+						  GNOME_STOCK_PIXMAP_QUIT),
+			_("Log out"));
 	gtk_menu_append (GTK_MENU (menu), menuitem);
 	gtk_signal_connect (GTK_OBJECT (menuitem), "activate", (GtkSignalFunc) panel_logout, 0);
+
 }
 
 static Menu *
-create_panel_menu (GtkWidget *window, char *menudir, int main_menu,
-		   MenuOrient orient)
+create_panel_menu (char *menudir, int main_menu,
+		   PanelOrientType orient)
 {
 	GtkWidget *pixmap;
 	Menu *menu;
 	GtkWidget *app_menu;
 	
-	char *pixmap_name;
+	char *pixmap_name = NULL;
 
 	menu = g_new(Menu,1);
 
 	if (main_menu)
 		switch(orient) {
-			case MENU_DOWN:
+			case ORIENT_DOWN:
 				pixmap_name = gnome_unconditional_pixmap_file ("gnome-menu-down.png");
 				break;
-			case MENU_UP:
+			case ORIENT_UP:
 				pixmap_name = gnome_unconditional_pixmap_file ("gnome-menu-up.png");
 				break;
-			case MENU_RIGHT:
+			case ORIENT_RIGHT:
 				pixmap_name = gnome_unconditional_pixmap_file ("gnome-menu-right.png");
 				break;
-			case MENU_LEFT:
+			case ORIENT_LEFT:
 				pixmap_name = gnome_unconditional_pixmap_file ("gnome-menu-left.png");
 				break;
 		}
@@ -603,25 +687,27 @@ create_panel_menu (GtkWidget *window, char *menudir, int main_menu,
 		/*this should not happen init root menu should have
 		  been called (unless there are no panels*/
 		if(!root_menu) {
-			root_menu = create_menu_at(window,menudir,0);
-			app_menu = create_menu_at (window, menudir, 1);
+			root_menu = create_menu_at(menudir,0,FALSE);
+			app_menu = create_menu_at (menudir, 1,FALSE);
 			add_special_entries (root_menu, app_menu);
 		}
 		menu->menu = root_menu;
 	} else {
-		menu->menu = create_menu_at (window, menudir, 0);
+		menu->menu = create_menu_at (menudir, 0, FALSE);
 	}
 	gtk_signal_connect (GTK_OBJECT (menu->button), "button_press_event",
 			    GTK_SIGNAL_FUNC (menu_button_press), menu);
 	gtk_signal_connect (GTK_OBJECT (menu->button), "destroy",
 			    GTK_SIGNAL_FUNC (destroy_menu), menu);
+	gtk_signal_connect (GTK_OBJECT (menu->menu), "deactivate",
+			    GTK_SIGNAL_FUNC (menu_deactivate), menu);
 
 	g_free (pixmap_name);
 	return menu;
 }
 
-GtkWidget *
-init_main_menu(GtkWidget *window)
+void
+init_main_menu(void)
 {
 	GtkWidget *app_menu;
 	char *menu_base = gnome_unconditional_datadir_file ("apps");
@@ -631,23 +717,23 @@ init_main_menu(GtkWidget *window)
 	g_free (menu_base);
 	if (!g_file_exists (menudir)) {
 		g_free (menudir);
-		return NULL;
+		return;
 	}
 
-	root_menu = create_menu_at(window,menudir,0);
-	app_menu = create_menu_at (window,menudir, 1);
+	root_menu = create_menu_at(menudir,0,FALSE);
+	app_menu = create_menu_at (menudir, 1,FALSE);
 	add_special_entries (root_menu, app_menu);
 	g_free (menudir);
 }
 
 Menu *
-create_menu_applet(GtkWidget *window, char *arguments, MenuOrient orient)
+create_menu_applet(char *arguments, PanelOrientType orient)
 {
 	Menu *menu;
 	int main_menu;
 
-	char *menu_base = gnome_unconditional_datadir_file ("apps");
 	char *this_menu;
+	char *menu_base = gnome_unconditional_datadir_file ("apps");
 
 	/*if null, let's put the main menu up*/
 	if (arguments == NULL)
@@ -667,7 +753,7 @@ create_menu_applet(GtkWidget *window, char *arguments, MenuOrient orient)
 	
 	if(!gnome_folder) {
 		gnome_folder = gnome_unconditional_pixmap_file
-					("gnome-folder-small.xpm");
+					("gnome-folder.png");
 		if (!g_file_exists (gnome_folder)) {
 			free (gnome_folder);
 			gnome_folder = NULL;
@@ -676,11 +762,13 @@ create_menu_applet(GtkWidget *window, char *arguments, MenuOrient orient)
 	
 	main_menu = (strcmp (arguments, ".") == 0);
 
-	menu = create_panel_menu (window, this_menu, main_menu, orient);
+	menu = create_panel_menu (this_menu, main_menu, orient);
 	menu->path=g_strdup(arguments);
 
 	gtk_object_set_user_data(GTK_OBJECT(menu->button),menu);
-
+	
+	g_free(menu_base);
+	g_free (this_menu);
 	return menu;
 }
 
@@ -699,31 +787,31 @@ set_show_small_icons(gpointer data, gpointer user_data)
 }
 
 void
-set_menu_applet_orient(Menu *menu, MenuOrient orient)
+set_menu_applet_orient(Menu *menu, PanelOrientType orient)
 {
 	GtkWidget *pixmap;
-	char *pixmap_name;
+	char *pixmap_name = NULL;
 
 	g_return_if_fail(menu!=NULL);
-	//g_return_if_fail(menu->path!=NULL);
+	g_return_if_fail(menu->path!=NULL);
 
 	menu->orient = orient;
 
 	if (strcmp (menu->path, ".") == 0)
 		switch (menu->orient) {
-			case MENU_DOWN:
+			case ORIENT_DOWN:
 				pixmap_name = gnome_unconditional_pixmap_file(
 					"gnome-menu-down.png");
 				break;
-			case MENU_UP:
+			case ORIENT_UP:
 				pixmap_name = gnome_unconditional_pixmap_file(
 					"gnome-menu-up.png");
 				break;
-			case MENU_RIGHT:
+			case ORIENT_RIGHT:
 				pixmap_name = gnome_unconditional_pixmap_file(
 					"gnome-menu-right.png");
 				break;
-			case MENU_LEFT:
+			case ORIENT_LEFT:
 				pixmap_name = gnome_unconditional_pixmap_file(
 					"gnome-menu-left.png");
 				break;
@@ -732,9 +820,10 @@ set_menu_applet_orient(Menu *menu, MenuOrient orient)
 		/*FIXME: these guys need arrows as well*/
 		pixmap_name = gnome_unconditional_pixmap_file ("gnome-folder.png");
 		
-	pixmap=GTK_BUTTON(menu->button)->child;
+	pixmap=GTK_BIN(menu->button)->child;
 	gtk_container_remove(GTK_CONTAINER(menu->button),pixmap);
-	gtk_widget_destroy(pixmap);
+	/*this is done by remove right?*/
+	/*gtk_widget_unref(pixmap);*/
 
 	/*make the pixmap*/
 	pixmap = gnome_pixmap_new_from_file_at_size (pixmap_name,
