@@ -16,16 +16,16 @@ extern "C" {
 #define PANEL_CELL_SIZE 10
 #define PANEL_MINIMUM_WIDTH 48
 
-#define PANEL_MAX 1000 /*10,000 long screen, FIXME! make dynamic*/
-
-#define PANEL_DRAWER_DROP_TARGET_SIZE 5
-
 #define PANEL_APPLET_PARENT_KEY "panel_applet_parent_key"
+#define PANEL_APPLET_ASSOC_PANEL_KEY "panel_applet_assoc_panel_key"
+#define PANEL_APPLET_FORBIDDEN_PANELS "panel_applet_forbidden_panels"
+#define PANEL_APPLET_DATA "panel_applet_data"
 
 typedef struct _PanelWidget		PanelWidget;
 typedef struct _PanelWidgetClass	PanelWidgetClass;
 
 typedef struct _AppletRecord		AppletRecord;
+typedef struct _AppletData		AppletData;
 typedef struct _DNDRecord		DNDRecord;
 typedef enum {
 	PANEL_HORIZONTAL,
@@ -54,16 +54,31 @@ typedef enum {
 	DROP_ZONE_LEFT,
 	DROP_ZONE_RIGHT
 } DrawerDropZonePos;
+typedef enum {
+	PANEL_SWITCH_MOVE,
+	PANEL_FREE_MOVE
+} PanelMovementType;
+typedef enum {
+	PANEL_BACK_NONE,
+	PANEL_BACK_COLOR,
+	PANEL_BACK_PIXMAP
+} PanelBackType;
 
-struct _AppletRecord
+struct _AppletData
 {
-	GtkWidget		*applet;
-	gint			cells;
+	GtkWidget *applet;
+	gint pos;
+	gint cells;
+
+	gint prevx;
+	gint prevy;
+	gint prevwidth;
+	gint prevheight;
 };
 
 struct _DNDRecord
 {
-	AppletRecord		*applet;
+	AppletData		*ad;
 	GtkWidget		*parent;
 };
 
@@ -71,6 +86,7 @@ struct _PanelWidget
 {
 	GtkWindow		window;
 
+	GtkWidget		*frame;
 	GtkWidget		*fixed;
 	GtkWidget		*table;
 	GtkWidget		*hidebutton_n;
@@ -78,28 +94,37 @@ struct _PanelWidget
 	GtkWidget		*hidebutton_w;
 	GtkWidget		*hidebutton_s;
 
-	AppletRecord		applets[PANEL_MAX];
-	gint	 		applet_count;
+	GList			*applet_list;
 
 	gint			size;
 	PanelOrientation	orient;
 	PanelSnapped		snapped;
 	PanelMode		mode;
 	PanelState		state;
-
-	gint			step_size;
-	gint			minimized_size;
-	gint			minimize_delay;
+	gboolean		fit_pixmap_bg;
 
 	gint			leave_notify_timer_tag;
 
-	GtkWidget		*currently_dragged_applet;
-	gint			currently_dragged_applet_pos;
+	AppletData		*currently_dragged_applet;
 
-	GtkWidget		*drawer_drop_zone;
 	DrawerDropZonePos	drawer_drop_zone_pos;
+	gint			x;
+	gint			y;
 
 	gint			thick;
+
+	gint			drawers_open; /* a count which can be used
+						 to block the autohide, until
+						 it is 0 .. it's set by the
+						 app not the widget*/
+	GtkWidget		*master_widget; /*the drawer button that is
+						  holding this panel*/
+
+	PanelBackType		back_type;
+	char                    *back_pixmap;
+	GdkColor		back_color;
+
+	gint			autohide_inhibit;
 };
 
 struct _PanelWidgetClass
@@ -113,6 +138,9 @@ struct _PanelWidgetClass
 			       PanelState state);
 	void (* applet_move) (PanelWidget *panel,
 			      GtkWidget *applet);
+	void (* applet_added) (PanelWidget *panel,
+			       GtkWidget *applet);
+	void (* applet_removed) (PanelWidget *panel);
 };
 
 guint		panel_widget_get_type		(void);
@@ -121,25 +149,35 @@ GtkWidget*	panel_widget_new		(gint size,
 						 PanelSnapped snapped,
 						 PanelMode mode,
 						 PanelState state,
-						 gint step_size,
-						 gint minimized_size,
-						 gint minimize_delay,
 						 gint pos_x,
 						 gint pos_y,
-						 DrawerDropZonePos
-						 	drop_zone_pos);
+						 DrawerDropZonePos drop_zone_pos,
+						 PanelBackType back_type,
+						 char *back_pixmap,
+						 gboolean fit_pixmap_bg,
+						 GdkColor *back_color);
+
 /*add an applet to the panel, preferably at position pos*/
 gint		panel_widget_add		(PanelWidget *panel,
 						 GtkWidget *applet,
 						 gint pos);
-/*move oldpos to newpos*/
+/*needs to be called for drawers after add*/
+void		panel_widget_add_forbidden	(PanelWidget *panel);
+
+/*move applet to newpos*/
 gint		panel_widget_move		(PanelWidget *panel,
-						 gint oldpos,
+						 GtkWidget *applet,
 						 gint pos);
 
 /*remove an applet from the panel*/
 gint		panel_widget_remove		(PanelWidget *panel,
 						 GtkWidget *applet);
+
+/*move applet to a different panel*/
+gint		panel_widget_reparent		(PanelWidget *old_panel,
+						 PanelWidget *new_panel,
+						 GtkWidget *applet,
+						 gint pos);
 /*return position of an applet*/
 gint		panel_widget_get_pos		(PanelWidget *panel,
 						 GtkWidget *applet);
@@ -155,32 +193,73 @@ void		panel_widget_foreach		(PanelWidget *panel,
   state to drawers and such*/
 void		panel_widget_restore_state	(PanelWidget *panel);
 
-/*initiate drag*/
+
+
+/*open and close drawers*/
+void		panel_widget_open_drawer	(PanelWidget *panel);
+void		panel_widget_close_drawer	(PanelWidget *panel);
+
+/*drag*/
 void		panel_widget_applet_drag_start	(PanelWidget *panel,
 						 GtkWidget *applet);
+void		panel_widget_applet_drag_end	(PanelWidget *panel);
 
-/*needed for corba*/
+/* needed for corba */
 void		panel_widget_applet_drag_start_no_grab(PanelWidget *panel,
 						       GtkWidget *applet);
 void		panel_widget_applet_drag_end_no_grab(PanelWidget *panel);
-int		panel_widget_applet_move_to_cursor(PanelWidget *panel);
+gint		panel_widget_applet_move_to_cursor(PanelWidget *panel);
 void		panel_widget_applet_move_use_idle(PanelWidget *panel);
 
-/*changing parameters*/
+/* changing parameters */
 void		panel_widget_change_params	(PanelWidget *panel,
 						 PanelOrientation orient,
 						 PanelSnapped snapped,
 						 PanelMode mode,
 						 PanelState state,
-						 gint step_size,
-						 gint minimized_size,
-						 gint minimize_delay,
 						 DrawerDropZonePos
-						 	drop_zone_pos);
+						 	drop_zone_pos,
+						 PanelBackType back_type,
+						 char *pixmap_name,
+						 gboolean fit_pixmap_bg,
+						 GdkColor *back_color);
 
-/*changing parameters (orient only)*/
+/* changing parameters (orient only) */
 void		panel_widget_change_orient	(PanelWidget *panel,
 						 PanelOrientation orient);
+/*changing parameters (dropzonepos only)*/
+void		panel_widget_change_drop_zone_pos(PanelWidget *panel,
+						  DrawerDropZonePos
+						 	drop_zone_pos);
+
+/*change global params*/
+void		panel_widget_change_global	(gint explicit_step,
+						 gint auto_step,
+						 gint drawer_step,
+						 gint minimized_size,
+						 gint minimize_delay,
+						 PanelMovementType move_type,
+						 gint disable_animations);
+
+/*popup the widget if it's popped down (autohide)*/
+void		panel_widget_pop_up		(PanelWidget *panel);
+
+/*queue a pop_down in autohide mode*/
+void		panel_widget_queue_pop_down	(PanelWidget *panel);
+
+void		panel_widget_enable_buttons	(PanelWidget *panel);
+void		panel_widget_disable_buttons	(PanelWidget *panel);
+
+void		panel_widget_set_drawer_pos	(PanelWidget *panel,
+						 gint x,
+						 gint y);
+
+/*get the number of applets*/
+gint		panel_widget_get_applet_count	(PanelWidget *panel);
+
+extern GList *panels;
+
+extern gint panel_applet_in_drag;
 
 #ifdef __cplusplus
 }
